﻿#!/usr/bin/python

from Bio.PDB import Structure,Model,Chain,Residue,Atom,DSSP, PDBParser, NeighborSearch, PDBIO, Vector
import pickle, os, sys
import argparse, openbabel
from numpy import array, zeros, append, arange
from math import ceil, sqrt, log, acos, pi, degrees
#import string
import matplotlib.pyplot as plt
#import pandas as pd
from modified_aa import *
#import matplotlib.axis as axi
#import matplotlib.ticker as ticker



def get_prot_atoms(plik_name, plik):
    prot = pars.get_structure('native', plik)
    prot_m = prot[0]
    prot_atoms = []
    ligand = []
    dssp = DSSP(prot_m, plik)
    for prot_chain in prot_m:
        for res in prot_chain:
            #print (res.get_id()[1], prot_chain.id)
            if "het=H_" not in repr(res) or res.resname in modified_aa:
                #try: 
                #    sasa = dssp[(prot_chain.id, res.get_id()[1])][3]
                    #print (sasa, type(sasa))
                #    if type(sasa) == type(2.3):
                #        if sasa > 0.1:
                for at in res.child_list:
                    if at.get_name()[0] != 'H':
                        prot_atoms.append(at)
                #except KeyError:
                #    print ('Warning!!!! DSSP program did not calculate sasa for chain %s residue %s' %(prot_chain, res.get_id()[1]))        
	    
            else:
                ligand.append(res)
	
  # szukanie liganda	
  #      for chain in prot_m:
   #         if chain.id != prot_chain:
		#print chain.id.strip(), prot_chain.strip()
        #        for res in chain:
         #           if "het=H_" in repr(res):
          #              if res not in ligand: ligand.append(res)
	
    #print (ligand, len(prot_atoms))
    ns = NeighborSearch(prot_atoms)
		  #ns=NeighborSearch(prot_atoms_list)
    return ns, prot, dssp
  
def get_ligand(pl_name):
    pl = pl_name.split('_whole')[0]+'.mol2'
    mol = open('niepodobne_Jul_cdHit_30_seqv2_recznie2/%s' %pl)
    lig_file = mol.readlines()
    return lig_file
  
def get_prot_at_type(n):
    aminoacids_side_chains = {'SER':{'OG':'mixS'}, 'CYS':{'SG':'nonPolarS'}, 'VAL':{'CG1':'nonPolarS', 'CG2':'nonPolarS'}, 'THR':{'CG2':'nonPolarS', 'OG1':'mixS'}, 'ILE':{'CG1':'nonPolarS', 'CG2':'nonPolarS', 'CD1':'nonPolarS'}, 'PRO':{'CG':'nonPolarS', 'CD':'nonPolarS'}, 'MET':{'CG':'nonPolarS', 'SD':'nonPolarS', 'CE':'nonPolarS'}, 'ASP':{'CG':'nonPolarS', 'OD1':'accepS', 'OD2':'accepS'}, 'ASN':{'CG':'nonPolarS', 'ND2':'donorS', 'OD1':'accepS'}, 'LEU':{'CG':'nonPolarS', 'CD1':'nonPolarS', 'CD2':'nonPolarS'}, 'LYS':{'CG':'nonPolarS','CD':'nonPolarS', 'CE':'nonPolarS', 'NZ':'donorS'}, 'GLU':{'CG':'nonPolarS', 'CD':'nonPolarS', 'OE1':'accepS', 'OE2':'accepS'}, 'GLN':{'CG':'nonPolarS', 'CD':'nonPolarS','OE1':'accepS', 'NE2':'donorS'}, 'ARG':{'CG':'nonPolarS', 'CD':'nonPolarS', 'CZ':'nonPolarS', 'NE':'donorS', 'NH1':'donorS', 'NH2':'donorS'}, 'HIS':{'CG':'nonPolarS', 'CD2':'nonPolarS', 'CE1':'nonPolarS', 'ND1':'mixS', 'NE2':'mixS'}, 'PHE':{'CG':'nonPolarS', 'CD1':'nonPolarS', 'CD2':'nonPolarS', 'CE1':'nonPolarS', 'CE2':'nonPolarS', 'CZ':'nonPolarS'}, 'TYR':{'CG':'nonPolarS', 'CD1':'nonPolarS', 'CD2':'nonPolarS', 'CE1':'nonPolarS', 'CE2':'nonPolarS', 'CZ':'nonPolarS', 'OH':'mixS'}, 'TRP':{'CG':'nonPolarS', 'CD1':'nonPolarS', 'CD2':'nonPolarS', 'CE2':'nonPolarS', 'CE3':'nonPolarS','CZ2':'nonPolarS', 'CZ3':'nonPolarS', 'CH2':'nonPolarS', 'NE1':'donorS'}}
   
    aminoacids_defined = {'CB' :'nonPolarS', 'CA':'nonPolarM', 'C':'nonPolarM', 'N':'donorM', 'O':'accepM'}
    
    prot_res_name = n.get_parent().get_resname()
    at_name = n.get_name().strip()
    #print prot_res, at_name
    
    if prot_res_name == 'PRO' and at_name == 'N': 
        prot_at_type = 'mixM'   
    elif at_name not in aminoacids_defined.keys():
        try: prot_at_type = aminoacids_side_chains[prot_res_name][at_name]
        except KeyError: prot_at_type = 'Unknown'
    else: 
        prot_at_type = aminoacids_defined[at_name]
	
    return prot_at_type
  
  

def create_mol_ligand(r, f, obConversion, mol):    
    s = Structure.Structure('ligand')
    m = Model.Model(0)
    ch = Chain.Chain('A')  
    s.add(m)
    m.add(ch)
    ch.add(r)
    
    # pdb file with ligand
    io = PDBIO()
    io.set_structure(s)
    file_name = f+'_'+str(r.resname)
    pdb_name = file_name+'.pdb.tmp' 
    io.save(pdb_name)
    
    obConversion.ReadFile(mol, pdb_name)  
    mol_name = file_name+'.mol2.tmp'  
    obConversion.WriteFile(mol, mol_name) 
    
    # mol file with ligand
    file_mol = open(mol_name, 'r')
    ligand_mol = file_mol.readlines()
    file_mol.close() 
    os.system("rm "+file_name+"*")
    return ligand_mol  

def get_complex_components(plik, ligand_names):
    plik_name = os.path.basename(plik)
    print (plik_name)
    ns, prot, sasa = get_prot_atoms(plik_name, plik)
    res = prot.get_residues()
    obConversion = openbabel.OBConversion()
    obConversion.SetInAndOutFormats("pdb", "mol2")
    mol = openbabel.OBMol()
    ligands = []
    lagand_names = []
    
    if len(ligand_names) > 0:
        ligand_names = ligand_names.split(',')	               
        for r in res:
	    het_flag = r.id[0][0]
            lig_name = r.resname.strip()
            chain = r.get_parent()
            res_id = chain.id + " " + str(r.id[1])
            if het_flag == 'H' and lig_name in ligand_names:
                print ("Loading conformer:", lig_name, res_id)   			  
                mol_l = create_mol_ligand(r, plik_name, obConversion, mol)
                ligands.append(mol_l)
                ligand_names.append(lig_name)
            else: 
                lig_name = '' 
               
    
    #without ligand list
    elif len(ligand_names) == 0:    
        for r in res:
            het_flag = r.id[0][0]
            lig_name = r.resname.strip()
            chain = r.get_parent()
            res_id = chain.id + " " + str(r.id[1])
            if het_flag == 'H':    
                if modified_aa.has_key(lig_name): lig_name = ''
                else:
                    print ("Loading conformer:", lig_name, res_id)
                    mol_l = create_mol_ligand(r, plik_name, obConversion, mol)						                
                    ligands.append(mol_l)
                    ligand_names.append(lig_name)
            else: pass            
    else: pass
  
    #check if ligand was found
    if len(lig_name) > 0: pass
    else:
        print ("LIGAND was not found!!! - Ligand has to have HETATM instead of ATOM")
        sys.exit(1)
    return ns, ligands, ligand_names, sasa

def ilwek (a,b):
  return (a[1]*b[2] - a[2]*b[1]) + (a[2]*b[0] - a[0]*b[2]) + (a[0]*b[1] - a[1]*b[0])

def calc_ang (one, two, three):
  a = one - two
  b = three - two
  an = Vector.norm(a)
  bn = Vector.norm(b)
  wekab = ilwek (a,b)
  sinab = wekab / (an*bn)
  cosab = (a*b)/(an*bn)
  ang = acos (cosab)
  if sinab<0: ang = ang+pi
  return degrees (ang)

def calc_energy(plik, ener, ns, lig_files, lig_namesi, ds):
    E = 0.0
    #for plik in pliki.readlines():
    #plik = plik.split()[0]
    #print (plik)
    #plik_name = plik.split('/')[-1]
    #prot_atm, ns = get_prot_atoms(plik_name, plik)
    #lg_file = get_ligand(plik_name)   
    #lig_name = plik_name.split('_')[2]
    #prot_atm, ns, lg_file, lig_name = get_complex_components(plik, ligand_names)
    for lg_file in lig_files:
        for line in lg_file:
            line_list = line.split()
            if any(lig_name in line for lig_name in lig_names) and not '.pdb' in line and len(line_list)>7 and not '**' in line:
	        lig_name = line_list[7], 
                coord = array([float(line_list[2]),float(line_list[3]),float(line_list[4])], "f")
                charge = line_list[-1]
                lig_t = round(float(charge),1)
            #if lig_t in lig_types.keys(): # count nr of lig types
             #   lig_types[lig_t] +=1
            #else: lig_types[lig_t] = 1
	    #print charge, 'charge'
                neighbors = ns.search(coord, 6.0, 'A')
	    #print neighbors
                for n in neighbors:
                    if n.get_name() !='OXT':
                        prot_atom_type = get_prot_at_type(n)
                        if prot_atom_type == 'Unknown': continue
                    else: continue
                    n_res = n.get_parent()
                    n_cha = n_res.get_parent()
                    sa = ds[(n_cha.id, n_res.get_id()[1])][3]
                    #print (sasa, type(sasa))
                    if type(sa) == type(2.3):
                        if sa > 0.7:
                            #print n_res.get_resname(), n_res.get_id(), "SASA duze!!!"
                            E = E + 1000.0
                            continue
                    else: pass
                    dis = sqrt(pow(float(line_list[2]) - n.get_coord()[0], 2) + pow(float(line_list[3]) - n.get_coord()[1], 2) + pow(float(line_list[4]) - n.get_coord()[2], 2))
                    
                    if n.get_name() != "CA":
                        if "CA" in n_res.child_dict:
                            on = n_res["CA"].get_vector()
                            tw = n.get_vector()
                            thr = Vector(coord)
                            #print type(on), type(tw), type(thr), tw, thr
                            ang = calc_ang(on, tw, thr)
                            if ang >=0.0 and ang <=20.0: angle = 20
                            elif ang >20.0 and ang <=40.0: angle = 40
                            elif ang >40.0 and ang <=60.0: angle = 60
                            elif ang >60.0 and ang <=80.0: angle = 80
                            elif ang >80.0 and ang <=100.0: angle = 100
                            elif ang >100.0 and ang <=120.0: angle = 120
                            elif ang >120.0 and ang <=140.0: angle = 140
                            elif ang >140.0 and ang <=160.0: angle = 160
                            elif ang >160.0 and ang <=180.0: angle = 180
                            elif ang >180.0 and ang <=200.0: angle = 200
                            elif ang >200.0 and ang <=220.0: angle = 220
                            elif ang >220.0 and ang <=240.0: angle = 240
                            elif ang >240.0 and ang <=260.0: angle = 260
                            elif ang >260.0 and ang <=280.0: angle = 280
                            elif ang >280.0 and ang <=300.0: angle = 300
                            elif ang >300.0 and ang <=320.0: angle = 320
                            elif ang >320.0 and ang <=340.0: angle = 340
                            elif ang >340.0 and ang <=360.0: angle = 360

                            wiez_ang_dis = "%s:%f_%i:%i" %(prot_atom_type, lig_t, ceil(dis), angle)
                        else: continue


                    else:
                        if "CB" in n.get_parent().child_dict:
                            on = n.get_parent()["CB"].get_vector()
                            tw = n.get_vector()
                            thr = Vector(coord)
                            #print type(on), type(tw), type(thr), tw, thr
                            ang = calc_ang(on, tw, thr)
                            if ang >=0.0 and ang <=20.0: angle = 20
                            elif ang >20.0 and ang <=40.0: angle = 40
                            elif ang >40.0 and ang <=60.0: angle = 60
                            elif ang >60.0 and ang <=80.0: angle = 80
                            elif ang >80.0 and ang <=100.0: angle = 100
                            elif ang >100.0 and ang <=120.0: angle = 120
                            elif ang >120.0 and ang <=140.0: angle = 140
                            elif ang >140.0 and ang <=160.0: angle = 160
                            elif ang >160.0 and ang <=180.0: angle = 180
                            elif ang >180.0 and ang <=200.0: angle = 200
                            elif ang >200.0 and ang <=220.0: angle = 220
                            elif ang >220.0 and ang <=240.0: angle = 240
                            elif ang >240.0 and ang <=260.0: angle = 260
                            elif ang >260.0 and ang <=280.0: angle = 280
                            elif ang >280.0 and ang <=300.0: angle = 300
                            elif ang >300.0 and ang <=320.0: angle = 320
                            elif ang >320.0 and ang <=340.0: angle = 340
                            elif ang >340.0 and ang <=360.0: angle = 360

                            wiez_ang_dis = "%s:%f_%i:%i" %(prot_atom_type, lig_t, ceil(dis), angle)
                        else: continue

                    #wiez = "%s:%f_%i" %(prot_atom_type, lig_t, int(ceil(dis)))
                    #print wiez
                    if ener.has_key(wiez_ang_dis):
                        #print ('JEST', ener[wiez])
                        E = E + ener[wiez_ang_dis] - 500.0
                    else: continue

        print str(lig_name)[2:5], lig_files.index(lg_file)+1, E
        E = 0.0

def read_conformers(conformers_file):
    file_mol = open(conformers_file, 'r')
    mol_l = file_mol.read()
    file_mol.close()    
    conformers = []  
    ligand_names = []
    conf = mol_l.split('@<TRIPOS>MOLECULE')
    del conf[0]
    for c in conf:    
        c_l = c.split('\n')
        conformers.append(c_l)
        for line in c_l:
            columns = line.split()
            if len(columns) > 6:
                #print columns[7][:3]
                if columns[7][:3] not in ligand_names:
                    ligand_names.append(columns[7])
                break
    return conformers, ligand_names
                    
                    
if __name__ == '__main__': 

    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group()
    group.add_argument("-c", "--complex",help = "protein-ligand complex as a pdb file" )
    parser.add_argument("-ln", "--names",help="Eg. NEM,CIR", default='' )
    group.add_argument("-p", "--prot", help = "protein as a pdb file")
   # group1 = parser.add_mutually_exclusive_group()
   # group1.add_argument("-c", "--complex",help = "protein-ligand complex as a pdb file" )
    parser.add_argument("-l", "--ligs", help = "ligands in mol2 format")
    parser.add_argument("-d", "--direc", help = "full path to the directory with prot-lig complexes in pdb format")
    args = parser.parse_args()
    
    if len(sys.argv) == 1:
        print parser.format_help() #prints help if no arguments
        sys.exit(1)

    
    pars = PDBParser()
    #pliki = open(args.list) 
    
    f = open("energy_protType_ligChar_bin.dis.ang_nosasa.dat", "rb")
    energy = pickle.load(f)
    
    print "Ligand_name Ligand nr  Energy"
    
    if args.complex != None:
        print args.complex
        ns, lg_file, lig_name, sas = get_complex_components(args.complex, args.names)
        calc_energy(args.complex, energy, ns, lg_file, lig_name, sas)
    
    elif args.prot != None and args.ligs != None: 
        print "Protein file - ", args.prot, ", ligands file - ", args.ligs
        plik_name = os.path.basename(args.prot)
        ns, prot, sas = get_prot_atoms(plik_name, args.prot)
        lg_files, lig_names = read_conformers(args.ligs)
        calc_energy(args.prot, energy, ns, lg_files, lig_names, sas)
   
    elif args.prot != None and args.ligs == None:
        print "Please write the ligands in mol2 format filename!!"
        sys.exit(1)
    elif args.prot == None and args.ligs != None:     
        print "Please write the protein in pdb format filename!!"
        sys.exit(1)
        
        
    
    #calc_energy(pliki, energy)
    
    
